#include <avr/io.h>
#include <avr/interrupt.h>
#define F_CPU 16000000UL
#include <util/delay.h>

//------------------------------------------
//Define pentru PWN pentru servo
#define SERVO_PORT		PORTD
#define SERVO_PIN		PIND
#define SERVO_DDR		DDRD
#define SERVO_PWM_PIN1 PD5
#define SERVO_PWM_PIN2 PD4

//------------------------------------------
//Define pentru modulul de senzor digitali
#define SENSOR_DDR		DDRC
#define SENSOR_PIN		PINC
#define SENSOR_PORT		PORTC
#define SENSOR1_PIN		PC2
#define SENSOR2_PIN		PC3
#define SENSOR3_PIN		PC1
#define SENSOR4_PIN		PC0
//------------------------------------------

//------------------------------------------
//Define pentru driverul de motor 
#define MOTOR_DDR		DDRB
#define MOTOR_PORT		PORTB
#define MOTOR_PIN		PINB
#define FORWARD_PIN		PB2	
#define REVERSE_PIN		PB1
#define MOTOR_MASK		( _BV(FORWARD_PIN) | _BV(REVERSE_PIN))
#define MOTOR_PWM_PIN	PB3
//------------------------------------------

//------------------------------------------
//Define modulul de senzor analogici
#define CONVERTOR_PIN	PA0
//------------------------------------------
#define EPS				120



int calibrated_center;
float a,b;

int ADC_get(int channel) {
  	char temp = ADMUX & (0xE0);
  	ADMUX = channel & 0x07;
  	ADMUX |= temp;
  	ADCSRA |= _BV(ADSC);
  	loop_until_bit_is_set(ADCSRA, ADIF);
  	return ADC;
}

void ADC_calibrate() {
	calibrated_center = 0;
	int i = 0, val;
	while (i < 5) {
	// merge cu aging
		val = ADC_get(0);
		_delay_ms(20);
		if (i > 0)
			calibrated_center = 0.4 * calibrated_center + 0.6 * val;
		else
			calibrated_center = val;
		i++;
	}
	i = 0;
	DDRD |= _BV(PD7);
	while (i < 10) {
		i++;
		PORTD ^=_BV(PD7);
		_delay_ms(100);
	}
	a = 0.8;
	b = 480 - calibrated_center * a;
}	

void ADC_init(void) {
  //	ADMUX |= _BV(REFS0);
  	ADCSRA = _BV(ADEN) | _BV(ADPS2) | _BV(ADPS1) | _BV(ADPS0) | _BV(ADSC);
}

void TIMER_init(void) {
  	TCCR1A |= _BV(COM1A1) | _BV(COM1B1) | _BV(WGM11);
  	TCCR1B |= _BV(WGM13) | _BV(WGM12) | _BV(CS11) | _BV(CS10);
  	ICR1 = 4999;
	SERVO_DDR |= _BV(SERVO_PWM_PIN1) | _BV(SERVO_PWM_PIN2);
	
  	TCCR0 |= _BV(WGM01) | _BV(WGM11) | _BV(COM01) | _BV(CS02) | _BV(CS00);
  	MOTOR_DDR |= _BV(MOTOR_PWM_PIN);
  	OCR0 = 0xFF;
}

//-------------------------------------------
// Motor
void MOTOR_init(void) {
  	MOTOR_DDR |= _BV(FORWARD_PIN) | _BV(REVERSE_PIN);
}

void MOTOR_forward(void) {
  	char reg = MOTOR_PORT & ~MOTOR_MASK;
  	MOTOR_PORT = reg | _BV(FORWARD_PIN);
}

void MOTOR_reverse(void) {
  	char reg = MOTOR_PORT & ~MOTOR_MASK;
  	MOTOR_PORT = reg | _BV(REVERSE_PIN);
}

void MOTOR_idle(void) {
  	MOTOR_PORT &= ~MOTOR_MASK;
}

void MOTOR_brake(void) {
  	MOTOR_PORT |= MOTOR_MASK; 
}

void MOTOR_decelerate(int i) {
  	OCR0 -= i * 10;
	if (OCR0 < 0)
		OCR0 = 0;
}

void MOTOR_accelerate(int i) {
  	OCR0 += i * 10;
	if (OCR0 > 0xFF)
		OCR0 = 0xFF;
}

void MOTOR_fullspeed() {
  	OCR0 = 0xFF;
}
//-------------------------------------------

//-------------------------------------------
// Direction
void SERVO_center() {
 	OCR1A = 480;
}

void SERVO_angle(int i) {
	OCR1A += i;
	if (OCR1A > 560)
		OCR1A = 560;
}

void SERVO_value(int i) {
	OCR1A = i;
}

void SERVO_left(int i) {
	OCR1A += i;
	if (OCR1A > 560)
		OCR1A = 560;
}

void SERVO_right(int i) {
	OCR1A -= i * 10;
	if (OCR1A < 400)
		OCR1A = 400;
}

void SERVO_fullRight() {
	OCR1A = 400;
}

void SERVO_halfRight() {
	OCR1A = 440;
}

void SERVO_fullLeft() {
	OCR1A = 560;
}

void SERVO_halfLeft() {
	OCR1A = 520;
}
//-------------------------------------------

//-----------------------------------------
// Sensor

void SENSOR_init() {
	SENSOR_DDR &= ~(_BV(SENSOR1_PIN) | _BV(SENSOR2_PIN) | _BV(SENSOR3_PIN) | _BV(SENSOR4_PIN));
	SENSOR_PORT = (_BV(SENSOR1_PIN) | _BV(SENSOR2_PIN) | _BV(SENSOR3_PIN) | _BV(SENSOR4_PIN));
}
	

char SENSOR_getState() {
	char pin = SENSOR_PIN;
	char result = 0;
	_delay_ms(4);
	if (pin != SENSOR_PIN)
		return 0xFF;
	if (!(pin & _BV(SENSOR1_PIN)))
		result |= (1 << 3);
	if (!(pin & _BV(SENSOR2_PIN)))
		result |= (1 << 2);
	if (!(pin & _BV(SENSOR3_PIN)))
		result |= (1 << 1);
	if (!(pin & _BV(SENSOR4_PIN)))
		result |= (1 << 0);
	return result;
}
//-------------------------------------------


void ROBOT_init() {
	TIMER_init();
	ADC_init();
	SENSOR_init();
	MOTOR_init();
}

int getAngle(int val) {
	return a * val - b;
}

int main() {

	ROBOT_init();
	SERVO_center();
	ADC_calibrate();
	MOTOR_forward();
	_delay_ms(5000);
//	MOTOR_decelerate(20);
	int val, angle;
	int ok;
  	while(1) {
		char state = SENSOR_getState();
		val = ADC_get(0);
		angle = getAngle(val);
		ok = 1;
		switch(state) {
		case 0xFF:
				ok = 0;
				break;
		case 15:
				MOTOR_brake();
				SERVO_center();
				_delay_ms(5);
				MOTOR_reverse();
				break;
		case 1:
				//SERVO_value(angle);
				MOTOR_forward();
				SERVO_left(15);
				break;
		case 3:
				//SERVO_value(angle);
				MOTOR_forward();
				SERVO_halfLeft();
				break;
		case 7:
				//SERVO_value(angle);
				MOTOR_forward();
				SERVO_fullLeft();
				break;
		case 8: 
				//SERVO_value(angle);
				MOTOR_forward();
				SERVO_right(15);
				break;
		case 9:
				MOTOR_forward();
				SERVO_center();
				break;
		
		case 12:
				//SERVO_value(angle);
				MOTOR_forward();
				SERVO_halfRight();
				break;
		case 14:
			//	SERVO_value(angle);
				MOTOR_forward();
				SERVO_fullRight();
				break;
		}
		if (ok)
			_delay_ms(5);	
		
  	}

	return 0;
}
